package org.zeroturnaround.javarebel.integration.spring;

import org.zeroturnaround.javarebel.ClassResourceSource;
import org.zeroturnaround.javarebel.IntegrationFactory;
import org.zeroturnaround.javarebel.LoggerFactory;
import org.zeroturnaround.javarebel.Plugin;
import org.zeroturnaround.javarebel.integration.spring.cbp.AbstractBeanFactoryCBP;
import org.zeroturnaround.javarebel.integration.spring.cbp.AnnotationBeanValidationConfigurationLoaderCBP;
import org.zeroturnaround.javarebel.integration.spring.cbp.CachedIntrospectionResultsCBP;
import org.zeroturnaround.javarebel.integration.spring.cbp.ClassPathBeanDefinitionScannerCBP;
import org.zeroturnaround.javarebel.integration.spring.cbp.ComponentScanBeanDefinitionParserCBP;
import org.zeroturnaround.javarebel.integration.spring.cbp.DefaultListableBeanFactoryCBP;
import org.zeroturnaround.javarebel.integration.spring.cbp.DefaultSingletonBeanRegistryCBP;
import org.zeroturnaround.javarebel.integration.spring.cbp.GlobalMethodSecurityBeanDefinitionParserCBP;
import org.zeroturnaround.javarebel.integration.spring.cbp.HttpSecurityBeanDefinitionParserCBP;
import org.zeroturnaround.javarebel.integration.spring.cbp.InjectionMetadataCacheCBP;
import org.zeroturnaround.javarebel.integration.spring.cbp.LocalVariableTableParameterNameDiscovererCBP;
import org.zeroturnaround.javarebel.integration.spring.cbp.PathMatchingResourcePatternResolverCBP;
import org.zeroturnaround.javarebel.integration.spring.cbp.PropertyResourceConfigurerCBP;
import org.zeroturnaround.javarebel.integration.spring.cbp.ResourceUtilsCBP;
import org.zeroturnaround.javarebel.integration.spring.cbp.XmlBeanDefinitionReaderCBP;

/**
 * JavaRebel Spring plugin that manages Spring metadata reloading.
 * 
 * @author Jevgeni Kabanov
 */
public class SpringPlugin implements Plugin{
  public void preinit() {

    ClassLoader cl = SpringPlugin.class.getClassLoader();

    /*
     * Disables Spring annotation cache for dependency method and field annotations.
     */
    IntegrationFactory.getInstance()
    .addIntegrationProcessor(
        cl,
        new String[] {
            "org.springframework.beans.factory.annotation.AutowiredAnnotationBeanPostProcessor",
            "org.springframework.context.annotation.CommonAnnotationBeanPostProcessor",
            "org.springframework.orm.jpa.support.PersistenceAnnotationBeanPostProcessor"
        },
        new InjectionMetadataCacheCBP());       

    /*
     * Disables Spring introspection cache for JavaBeans Introspector results used in
     * resolving XML dependencies.
     */
    IntegrationFactory.getInstance()
    .addIntegrationProcessor(
        cl,
        "org.springframework.beans.CachedIntrospectionResults", 
        new CachedIntrospectionResultsCBP());            
    
    /*
     * Registers the SpringBeanReloader for each bean factory 
     * and call checkAndReload() on all classes being configured.
     */
    IntegrationFactory.getInstance()
    .addIntegrationProcessor(
        cl,
        "org.springframework.beans.factory.support.AbstractBeanFactory", 
        new AbstractBeanFactoryCBP());       

    /*
     * Disables flushing singleton cache preserving all singleton instances.
     */
    IntegrationFactory.getInstance()
    .addIntegrationProcessor(
        cl,
        "org.springframework.beans.factory.support.DefaultSingletonBeanRegistry", 
        new DefaultSingletonBeanRegistryCBP());      

    /*
     * Registers each XML bean resource with the SpringBeanReloader.
     */
    IntegrationFactory.getInstance()
    .addIntegrationProcessor(
        cl,
        "org.springframework.beans.factory.xml.XmlBeanDefinitionReader", 
        new XmlBeanDefinitionReaderCBP());     

    /*
     * Disables compatibility check for annotation-configured components 
     * found in classpath (needed from Spring 2.5.5)
     */
    IntegrationFactory.getInstance()
    .addIntegrationProcessor(
        cl,
        "org.springframework.context.annotation.ClassPathBeanDefinitionScanner", 
        new ClassPathBeanDefinitionScannerCBP());

    /*
     * Removes the bean definition to prevent overriding messages 
     * and duplicate exceptions.
     */
    IntegrationFactory.getInstance()
    .addIntegrationProcessor(
        cl,
        "org.springframework.beans.factory.support.DefaultListableBeanFactory", 
        new DefaultListableBeanFactoryCBP());

    /*
     * Avoids error "Duplicate <global-method-security> detected" during a
     * reload of Spring Security bean definitions.
     */
    IntegrationFactory.getInstance()
    .addIntegrationProcessor(
        cl,
        "org.springframework.security.config.GlobalMethodSecurityBeanDefinitionParser",
        new GlobalMethodSecurityBeanDefinitionParserCBP());

    /*
     * Avoids error "Duplicate <http> element detected" during a reload of
     * Spring Security bean definitions.
     */
    IntegrationFactory.getInstance()
    .addIntegrationProcessor(
        cl,
        "org.springframework.security.config.HttpSecurityBeanDefinitionParser",
        new HttpSecurityBeanDefinitionParserCBP());

    /*
     * Scans the classpath for new components.
     */
    IntegrationFactory.getInstance()
    .addIntegrationProcessor(
        cl,
        "org.springframework.context.annotation.ComponentScanBeanDefinitionParser",
        new ComponentScanBeanDefinitionParserCBP());    

    /*
     * Applies property placeholder and override configurers for beans reconfigured. 
     */
    IntegrationFactory.getInstance()
    .addIntegrationProcessor(
        cl,
        "org.springframework.beans.factory.config.PropertyResourceConfigurer",
        new PropertyResourceConfigurerCBP());
    
    /*
     * Disables Validator cache.
     */
    IntegrationFactory.getInstance()
    .addIntegrationProcessor(
        cl,
        "org.springmodules.validation.bean.conf.loader.annotation.AnnotationBeanValidationConfigurationLoader", 
        new AnnotationBeanValidationConfigurationLoaderCBP());  

    /*
     * Refresh method parameter names after class is reloaded. 
     */
    IntegrationFactory.getInstance()
    .addIntegrationProcessor(
        cl,
        "org.springframework.core.LocalVariableTableParameterNameDiscoverer", 
        new LocalVariableTableParameterNameDiscovererCBP());  
    
    /*
     * Modify classpath* resolution
     */
    IntegrationFactory.getInstance()
    .addIntegrationProcessor(
        cl,
        "org.springframework.core.io.support.PathMatchingResourcePatternResolver", 
        new PathMatchingResourcePatternResolverCBP());  
    
    /*
     * Fix ResourceUtils in JBoss 5
     */
    IntegrationFactory.getInstance()
    .addIntegrationProcessor(
        cl,
        "org.springframework.util.ResourceUtils", 
        new ResourceUtilsCBP());  
  }

  public boolean checkDependencies(ClassLoader cl, ClassResourceSource crs) {    
    boolean springPresent = crs.getClassResource("org.springframework.beans.factory.support.AbstractBeanFactory") != null;
    boolean spring20Present = crs.getClassResource("org.springframework.beans.factory.annotation.Configurable") != null;
    
    if (springPresent && !spring20Present) {
      LoggerFactory.getInstance().echo();
      LoggerFactory.getInstance().echo("JavaRebel: Spring 1.x is not supported by JavaRebel Spring plugin.");
      LoggerFactory.getInstance().echo("JavaRebel: Please use 2.x or later.");
      LoggerFactory.getInstance().echo();
    }
    
    return springPresent && spring20Present;
  }

  public String getId() {
    return "spring_plugin";
  }
  
  public String getName() {
    return "Spring Framework Plugin";
  }
  
  public String getDescription() {
    return "Supports adding new beans and adding new bean dependencies using annotations or XML. " +
    		   "Singletons will be reconfigured after the change. " +
    		   "It also supports adding or changing Spring MVC controllers or handlers.";
  }
  
  public String getAuthor() {
    return null;
  }
  
  public String getWebsite() {
    return null;
  }
}
